@@ -72,11 +72,14 @@ View::View(Compositor *compositor)
 
 QOpenGLTexture *View::getTexture()
 {
-    if (advance()) {
-        QWaylandBufferRef buf = currentBuffer();
+    bool newContent = advance();
+    QWaylandBufferRef buf = currentBuffer();
+    if (!buf.hasContent())
+        m_texture = nullptr;
+    if (newContent) {
         m_texture = buf.toOpenGLTexture();
         if (surface()) {
-            m_size = surface()->size();
+            m_size = surface()->destinationSize();
             m_origin = buf.origin() == QWaylandSurface::OriginTopLeft
                     ? QOpenGLTextureBlitter::OriginTopLeft
                     : QOpenGLTextureBlitter::OriginBottomLeft;
@@ -93,7 +96,7 @@ QOpenGLTextureBlitter::Origin View::textureOrigin() const
 
 QSize View::size() const
 {
-    return surface() ? surface()->size() : m_size;
+    return surface() ? surface()->destinationSize() : m_size;
 }
 
 bool View::isCursor() const
@@ -263,7 +266,7 @@ void Compositor::viewAnimationDone()
 
 View * Compositor::findView(const QWaylandSurface *s) const
 {
-    Q_FOREACH (View* view, m_views) {
+    for (View* view : m_views) {
         if (view->surface() == s)
             return view;
     }
@@ -440,25 +443,25 @@ void Compositor::handleMouseEvent(QWaylandView *target, QMouseEvent *me)
         closePopups();
     }
 
-    QWaylandSeat *input = defaultSeat();
+    QWaylandSeat *seat = defaultSeat();
     QWaylandSurface *surface = target ? target->surface() : nullptr;
     switch (me->type()) {
         case QEvent::MouseButtonPress:
-            input->sendMousePressEvent(me->button());
-            if (surface != input->keyboardFocus()) {
+            seat->sendMousePressEvent(me->button());
+            if (surface != seat->keyboardFocus()) {
                 if (surface == nullptr
                         || surface->role() == QWaylandWlShellSurface::role()
                         || surface->role() == QWaylandXdgSurfaceV5::role()
                         || surface->role() == QWaylandXdgPopupV5::role()) {
-                    input->setKeyboardFocus(surface);
+                    seat->setKeyboardFocus(surface);
                 }
             }
             break;
     case QEvent::MouseButtonRelease:
-         input->sendMouseReleaseEvent(me->button());
+         seat->sendMouseReleaseEvent(me->button());
          break;
     case QEvent::MouseMove:
-        input->sendMouseMoveEvent(target, me->localPos(), me->globalPos());
+        seat->sendMouseMoveEvent(target, me->localPos(), me->globalPos());
     default:
         break;
     }
@@ -542,6 +545,6 @@ void Compositor::raise(View *view)
         int source = endPos + 1 + i;
         int dest = startPos + i;
         for (int j = source; j > dest; j--)
-            m_views.swap(j, j-1);
+            m_views.swapItemsAt(j, j-1);
     }
 }
