@@ -5,7 +5,7 @@
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -14,24 +14,14 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** General Public License version 3 or (at your option) any later version
+** approved by the KDE Free Qt Foundation. The licenses are as published by
+** the Free Software Foundation and appearing in the file LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -52,11 +42,14 @@
 #include <QtWaylandCompositor/private/qwlclientbufferintegration_p.h>
 #include <QtWaylandCompositor/private/qwaylandsurface_p.h>
 
+#if QT_CONFIG(opengl)
+#  include <QtGui/QOpenGLTexture>
+#  include <QtGui/QOpenGLFunctions>
+#endif
+
 #include <QtGui/QKeyEvent>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QScreen>
-#include <QtGui/QOpenGLFunctions>
-#include <QtGui/QOpenGLTexture>
 
 #include <QtQuick/QSGSimpleTextureNode>
 #include <QtQuick/QQuickWindow>
@@ -64,7 +57,7 @@
 #include <QtCore/QMutexLocker>
 #include <QtCore/QMutex>
 
-#include <wayland-server.h>
+#include <wayland-server-core.h>
 #include <QThread>
 
 #ifndef GL_TEXTURE_EXTERNAL_OES
@@ -73,6 +66,7 @@
 
 QT_BEGIN_NAMESPACE
 
+#if QT_CONFIG(opengl)
 static const struct {
     const char * const vertexShaderSourceFile;
     const char * const fragmentShaderSourceFile;
@@ -83,7 +77,7 @@ static const struct {
     QSGMaterialType materialType;
 } bufferTypes[] = {
     // BufferFormatEgl_Null
-    { "", "", 0, 0, false, 0, {} },
+    { "", "", 0, 0, false, {}, {} },
 
     // BufferFormatEgl_RGB
     {
@@ -259,6 +253,7 @@ void QWaylandBufferMaterial::ensureTextures(int count)
         m_textures << nullptr;
     }
 }
+#endif // QT_CONFIG(opengl)
 
 QMutex *QWaylandQuickItemPrivate::mutex = nullptr;
 
@@ -284,10 +279,12 @@ public:
         if (m_ref.hasBuffer()) {
             if (buffer.isSharedMemory()) {
                 m_sgTex = surfaceItem->window()->createTextureFromImage(buffer.image());
-                if (m_sgTex) {
+#if QT_CONFIG(opengl)
+                if (m_sgTex)
                     m_sgTex->bind();
-                }
+#endif
             } else {
+#if QT_CONFIG(opengl)
                 QQuickWindow::CreateTextureOptions opt;
                 QWaylandQuickSurface *surface = qobject_cast<QWaylandQuickSurface *>(surfaceItem->surface());
                 if (surface && surface->useTextureAlpha()) {
@@ -295,7 +292,12 @@ public:
                 }
 
                 auto texture = buffer.toOpenGLTexture();
-                m_sgTex = surfaceItem->window()->createTextureFromId(texture->textureId() , QSize(surfaceItem->width(), surfaceItem->height()), opt);
+                GLuint textureId = texture->textureId();
+                auto size = surface->bufferSize();
+                m_sgTex = surfaceItem->window()->createTextureFromNativeObject(QQuickWindow::NativeObjectTexture, &textureId, 0, size, opt);
+#else
+                qCWarning(qLcWaylandCompositor) << "Without OpenGL support only shared memory textures are supported";
+#endif
             }
         }
         emit textureChanged();
@@ -414,7 +416,14 @@ QWaylandSurface *QWaylandQuickItem::surface() const
 void QWaylandQuickItem::setSurface(QWaylandSurface *surface)
 {
     Q_D(QWaylandQuickItem);
+    QWaylandSurface *oldSurf = d->view->surface();
+    QWaylandCompositor *oldComp = d->view->surface() ? d->view->surface()->compositor() : nullptr;
     d->view->setSurface(surface);
+    QWaylandCompositor *newComp = d->view->surface() ? d->view->surface()->compositor() : nullptr;
+    if (oldComp != newComp)
+        emit compositorChanged();
+    if (oldSurf != surface)
+        emit surfaceChanged();
     update();
 }
 
@@ -598,13 +607,17 @@ void QWaylandQuickItem::wheelEvent(QWheelEvent *event)
 {
     Q_D(QWaylandQuickItem);
     if (d->shouldSendInputEvents()) {
-        if (!inputRegionContains(event->posF())) {
+        if (!inputRegionContains(event->position())) {
             event->ignore();
             return;
         }
 
         QWaylandSeat *seat = compositor()->seatFor(event);
-        seat->sendMouseWheelEvent(event->orientation(), event->delta());
+        // TODO: fix this to send a single event, when diagonal scrolling is supported
+        if (event->angleDelta().x() != 0)
+            seat->sendMouseWheelEvent(Qt::Horizontal, event->angleDelta().x());
+        if (event->angleDelta().y() != 0)
+            seat->sendMouseWheelEvent(Qt::Vertical, event->angleDelta().y());
     } else {
         event->ignore();
     }
@@ -724,6 +737,7 @@ void QWaylandQuickItem::handleSubsurfaceAdded(QWaylandSurface *childSurface)
         childItem->setVisible(true);
         childItem->setParentItem(this);
         connect(childSurface, &QWaylandSurface::subsurfacePositionChanged, childItem, &QWaylandQuickItem::handleSubsurfacePosition);
+        connect(childSurface, &QWaylandSurface::destroyed, childItem, &QObject::deleteLater);
     } else {
         bool success = QMetaObject::invokeMethod(d->subsurfaceHandler, "handleSubsurfaceAdded", Q_ARG(QWaylandSurface *, childSurface));
         if (!success)
@@ -821,7 +835,15 @@ void QWaylandQuickItem::setOutput(QWaylandOutput *output)
 }
 
 /*!
- * \property QWaylandQuickItem::isBufferLocked
+ * \qmlproperty bool QtWaylandCompositor::WaylandQuickItem::bufferLocked
+ *
+ * This property holds whether the item's buffer is currently locked. As long as
+ * the buffer is locked, it will not be released and returned to the client.
+ *
+ * The default is false.
+ */
+/*!
+ * \property QWaylandQuickItem::bufferLocked
  *
  * This property holds whether the item's buffer is currently locked. As long as
  * the buffer is locked, it will not be released and returned to the client.
@@ -886,7 +908,7 @@ void QWaylandQuickItem::handleSurfaceChanged()
     if (d->oldSurface) {
         disconnect(d->oldSurface.data(), &QWaylandSurface::hasContentChanged, this, &QWaylandQuickItem::surfaceMappedChanged);
         disconnect(d->oldSurface.data(), &QWaylandSurface::parentChanged, this, &QWaylandQuickItem::parentChanged);
-        disconnect(d->oldSurface.data(), &QWaylandSurface::sizeChanged, this, &QWaylandQuickItem::updateSize);
+        disconnect(d->oldSurface.data(), &QWaylandSurface::destinationSizeChanged, this, &QWaylandQuickItem::updateSize);
         disconnect(d->oldSurface.data(), &QWaylandSurface::bufferScaleChanged, this, &QWaylandQuickItem::updateSize);
         disconnect(d->oldSurface.data(), &QWaylandSurface::configure, this, &QWaylandQuickItem::updateBuffer);
         disconnect(d->oldSurface.data(), &QWaylandSurface::redraw, this, &QQuickItem::update);
@@ -903,7 +925,7 @@ void QWaylandQuickItem::handleSurfaceChanged()
     if (QWaylandSurface *newSurface = d->view->surface()) {
         connect(newSurface, &QWaylandSurface::hasContentChanged, this, &QWaylandQuickItem::surfaceMappedChanged);
         connect(newSurface, &QWaylandSurface::parentChanged, this, &QWaylandQuickItem::parentChanged);
-        connect(newSurface, &QWaylandSurface::sizeChanged, this, &QWaylandQuickItem::updateSize);
+        connect(newSurface, &QWaylandSurface::destinationSizeChanged, this, &QWaylandQuickItem::updateSize);
         connect(newSurface, &QWaylandSurface::bufferScaleChanged, this, &QWaylandQuickItem::updateSize);
         connect(newSurface, &QWaylandSurface::configure, this, &QWaylandQuickItem::updateBuffer);
         connect(newSurface, &QWaylandSurface::redraw, this, &QQuickItem::update);
@@ -992,7 +1014,7 @@ void QWaylandQuickItem::updateSize()
 
     QSize size(0, 0);
     if (surface())
-        size = surface()->size() * (d->scaleFactor() / surface()->bufferScale());
+        size = surface()->destinationSize() * d->scaleFactor();
 
     setImplicitSize(size.width(), size.height());
     if (d->sizeFollowsSurface)
@@ -1053,6 +1075,14 @@ bool QWaylandQuickItem::inputRegionContains(const QPointF &localPosition)
     return const_cast<const QWaylandQuickItem *>(this)->inputRegionContains(localPosition);
 }
 
+/*!
+ * \qmlmethod point WaylandQuickItem::mapToSurface(point point)
+ *
+ * Maps the given \a point in this item's coordinate system to the equivalent
+ * point within the Wayland surface's coordinate system, and returns the mapped
+ * coordinate.
+ */
+
 /*!
  * Maps the given \a point in this item's coordinate system to the equivalent
  * point within the Wayland surface's coordinate system, and returns the mapped
@@ -1061,15 +1091,41 @@ bool QWaylandQuickItem::inputRegionContains(const QPointF &localPosition)
 QPointF QWaylandQuickItem::mapToSurface(const QPointF &point) const
 {
     Q_D(const QWaylandQuickItem);
-    if (!surface() || surface()->size().isEmpty())
+    if (!surface() || surface()->destinationSize().isEmpty())
         return point / d->scaleFactor();
 
-    qreal xScale = width() / surface()->size().width() * surface()->bufferScale();
-    qreal yScale = height() / surface()->size().height() * surface()->bufferScale();
+    qreal xScale = width() / surface()->destinationSize().width();
+    qreal yScale = height() / surface()->destinationSize().height();
 
     return QPointF(point.x() / xScale, point.y() / yScale);
 }
 
+/*!
+ * \qmlmethod point WaylandQuickItem::mapFromSurface(point point)
+ * \since 5.13
+ *
+ * Maps the given \a point in the Wayland surfaces's coordinate system to the equivalent
+ * point within this item's coordinate system, and returns the mapped coordinate.
+ */
+
+/*!
+ * Maps the given \a point in the Wayland surfaces's coordinate system to the equivalent
+ * point within this item's coordinate system, and returns the mapped coordinate.
+ *
+ * \since 5.13
+ */
+QPointF QWaylandQuickItem::mapFromSurface(const QPointF &point) const
+{
+    Q_D(const QWaylandQuickItem);
+    if (!surface() || surface()->destinationSize().isEmpty())
+        return point * d->scaleFactor();
+
+    qreal xScale = width() / surface()->destinationSize().width();
+    qreal yScale = height() / surface()->destinationSize().height();
+
+    return QPointF(point.x() * xScale, point.y() * yScale);
+}
+
 /*!
  * \qmlproperty bool QtWaylandCompositor::WaylandQuickItem::sizeFollowsSurface
  *
@@ -1135,9 +1191,11 @@ QVariant QWaylandQuickItem::inputMethodQuery(Qt::InputMethodQuery query, QVarian
 */
 
 /*!
-    Returns true if the item is hidden, though the texture
+    \property QWaylandQuickItem::paintEnabled
+
+    Holds \c true if the item is hidden, though the texture
     is still updated. As opposed to hiding the item by
-    setting \l{Item::visible}{visible} to \c false, setting this property to \c false
+    setting \l{QQuickItem::}{visible} to \c false, setting this property to \c false
     will not prevent mouse or keyboard input from reaching item.
 */
 bool QWaylandQuickItem::paintEnabled() const
@@ -1149,10 +1207,28 @@ bool QWaylandQuickItem::paintEnabled() const
 void QWaylandQuickItem::setPaintEnabled(bool enabled)
 {
     Q_D(QWaylandQuickItem);
-    d->paintEnabled = enabled;
+
+    if (enabled != d->paintEnabled) {
+        d->paintEnabled = enabled;
+        emit paintEnabledChanged();
+    }
+
     update();
 }
 
+/*!
+    \qmlproperty  bool QtWaylandCompositor::WaylandQuickItem::touchEventsEnabled
+
+    This property holds \c true if touch events are forwarded to the client
+    surface, \c false otherwise.
+*/
+
+/*!
+    \property QWaylandQuickItem::touchEventsEnabled
+
+    This property holds \c true if touch events are forwarded to the client
+    surface, \c false otherwise.
+*/
 bool QWaylandQuickItem::touchEventsEnabled() const
 {
     Q_D(const QWaylandQuickItem);
@@ -1245,7 +1321,7 @@ void QWaylandQuickItem::updateInputMethod(Qt::InputMethodQueries queries)
  * If an animation is started, bufferLocked should be set to ensure the item keeps its content
  * until the animation finishes
  *
- * \sa isBufferLocked
+ * \sa bufferLocked
  */
 
 /*!
@@ -1258,7 +1334,7 @@ void QWaylandQuickItem::updateInputMethod(Qt::InputMethodQueries queries)
  * If an animation is started, bufferLocked should be set to ensure the item keeps its content
  * until the animation finishes
  *
- * \sa QWaylandQuickkItem::bufferLocked
+ * \sa QWaylandQuickItem::bufferLocked
  */
 
 QSGNode *QWaylandQuickItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *data)
@@ -1270,7 +1346,7 @@ QSGNode *QWaylandQuickItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
     if (d->view->isBufferLocked() && !bufferHasContent && d->paintEnabled)
         return oldNode;
 
-    if (!bufferHasContent || !d->paintEnabled) {
+    if (!bufferHasContent || !d->paintEnabled || !surface()) {
         delete oldNode;
         return nullptr;
     }
@@ -1280,7 +1356,11 @@ QSGNode *QWaylandQuickItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
     const QRectF rect = invertY ? QRectF(0, height(), width(), -height())
                                 : QRectF(0, 0, width(), height());
 
-    if (ref.isSharedMemory() || bufferTypes[ref.bufferFormatEgl()].canProvideTexture) {
+    if (ref.isSharedMemory()
+#if QT_CONFIG(opengl)
+            || bufferTypes[ref.bufferFormatEgl()].canProvideTexture
+#endif
+    ) {
         // This case could covered by the more general path below, but this is more efficient (especially when using ShaderEffect items).
         QSGSimpleTextureNode *node = static_cast<QSGSimpleTextureNode *>(oldNode);
 
@@ -1301,46 +1381,53 @@ QSGNode *QWaylandQuickItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
         d->provider->setSmooth(smooth());
         node->setRect(rect);
 
-        return node;
-    } else {
-        Q_ASSERT(!d->provider);
-
-        QSGGeometryNode *node = static_cast<QSGGeometryNode *>(oldNode);
-
-        if (!node) {
-            node = new QSGGeometryNode;
-            d->newTexture = true;
-        }
-
-        QSGGeometry *geometry = node->geometry();
-        QWaylandBufferMaterial *material = static_cast<QWaylandBufferMaterial *>(node->material());
-
-        if (!geometry)
-            geometry = new QSGGeometry(QSGGeometry::defaultAttributes_TexturedPoint2D(), 4);
-
-        if (!material)
-            material = new QWaylandBufferMaterial(ref.bufferFormatEgl());
-
-        if (d->newTexture) {
-            d->newTexture = false;
-            for (int plane = 0; plane < bufferTypes[ref.bufferFormatEgl()].planeCount; plane++)
-                if (auto texture = ref.toOpenGLTexture(plane))
-                    material->setTextureForPlane(plane, texture);
-            material->bind();
-        }
-
-        QSGGeometry::updateTexturedRectGeometry(geometry, rect, QRectF(0, 0, 1, 1));
-
-        node->setGeometry(geometry);
-        node->setFlag(QSGNode::OwnsGeometry, true);
-
-        node->setMaterial(material);
-        node->setFlag(QSGNode::OwnsMaterial, true);
+        qreal scale = surface()->bufferScale();
+        QRectF source = surface()->sourceGeometry();
+        node->setSourceRect(QRectF(source.topLeft() * scale, source.size() * scale));
 
         return node;
     }
 
-    Q_UNREACHABLE();
+#if QT_CONFIG(opengl)
+    Q_ASSERT(!d->provider);
+
+    QSGGeometryNode *node = static_cast<QSGGeometryNode *>(oldNode);
+
+    if (!node) {
+        node = new QSGGeometryNode;
+        d->newTexture = true;
+    }
+
+    QSGGeometry *geometry = node->geometry();
+    QWaylandBufferMaterial *material = static_cast<QWaylandBufferMaterial *>(node->material());
+
+    if (!geometry)
+        geometry = new QSGGeometry(QSGGeometry::defaultAttributes_TexturedPoint2D(), 4);
+
+    if (!material)
+        material = new QWaylandBufferMaterial(ref.bufferFormatEgl());
+
+    if (d->newTexture) {
+        d->newTexture = false;
+        for (int plane = 0; plane < bufferTypes[ref.bufferFormatEgl()].planeCount; plane++)
+            if (auto texture = ref.toOpenGLTexture(plane))
+                material->setTextureForPlane(plane, texture);
+        material->bind();
+    }
+
+    QSGGeometry::updateTexturedRectGeometry(geometry, rect, QRectF(0, 0, 1, 1));
+
+    node->setGeometry(geometry);
+    node->setFlag(QSGNode::OwnsGeometry, true);
+
+    node->setMaterial(material);
+    node->setFlag(QSGNode::OwnsMaterial, true);
+
+    return node;
+#else
+    qCWarning(qLcWaylandCompositor) << "Without OpenGL support only shared memory textures are supported";
+    return nullptr;
+#endif // QT_CONFIG(opengl)
 }
 
 void QWaylandQuickItem::setTouchEventsEnabled(bool enabled)
